@@ -1,10 +1,11 @@
 package IO::ReadHandle::Include;
 
-use 5.006;
+use v5.10.0;
 use strict;
 use warnings;
 
 use Carp;
+use List::Util qw(none);
 use Path::Class qw(file);
 use Scalar::Util qw(blessed reftype);
 use Symbol qw(gensym);
@@ -18,11 +19,11 @@ facility
 
 =head1 VERSION
 
-Version 1.0.0
+Version 1.2.1
 
 =cut
 
-use version; our $VERSION = version->declare('v1.0.0');
+use version; our $VERSION = version->declare('v1.2.1');
 
 =head1 SYNOPSIS
 
@@ -71,8 +72,16 @@ The include directives are identified through a regular expression
 If the text read from the source file matches the regular expression,
 then, in the output, the part of the text matching the regular
 expression is replaced with the contents of the identified include
-file.  This works recursively: The included file can itself include
-other files, using the same format for include directives.
+file, if that include file exists.  This works recursively: The
+included file can itself include other files, using the same format
+for include directives.  If an include file does not exist, then the
+include directive naming that file is not replaced.
+
+An include file cannot recursively include itself, because that leads
+to an infinite loop.  If such an include directive detected, then it
+is not replaced.  It is not a problem if a particular file is included
+multiple times, as long as each next include of that file begins after
+the previous include has completed.
 
 The include file is identified by the text corresponding to a
 particular capture group (C<< (?<include>...) >> or C<$1>) of the
@@ -116,7 +125,7 @@ not always be recognized.
 This works well for the L<CORE::readline|perlfunc/readline> function,
 for the L</getline> and L</getlines> methods, and for the angle
 brackets operator (C<< <$ih> >>), which read text up to and including
-the an input record separator (or the end of the data, whichever comes
+the input record separator (or the end of the data, whichever comes
 first).
 
 =head2 INCLUDE DIRECTIVES AND THE READ FUNCTION
@@ -127,12 +136,12 @@ text does not necessarily end with the input record separator, so it
 might end in the middle of an include directive, and then the include
 directive cannot be recognized.
 
-To resolve this problem, the L</read> function/method by default
-quietly read beyond the requested number of characters until the next
-input record separator or the end of the data is seen, so it can
-properly detect and resolve any include directives.  It then returns
-only up to the requested number of characters, and remembers the
-remainder for the next call.
+To resolve this problem, the L</read> function/method when called on
+an IO::ReadHandle::Include object by default quietly read beyond the
+requested number of characters until the next input record separator
+or the end of the data is seen, so it can properly detect and resolve
+any include directives.  It then returns only up to the requested
+number of characters, and remembers the remainder for the next call.
 
 This means that if the source file or an include file contains no
 input record separator at all and is read using the L</read>
@@ -149,7 +158,7 @@ cannot apply to any include directives that were resolved while
 processing the L</read> call that produced the text that indicates the
 need to change the transformation.
 
-In such a case, use the L</set_read_by_lines> method to indicate that
+In such a case, use the L</set_read_by_line> method to indicate that
 you want L</read> to return text that does not extend beyond the first
 input record separator -- i.e., at most one line of text.  You may
 then get fewer characters from a call to L</read> than you asked for,
@@ -281,7 +290,8 @@ sub CLOSE {
   # close any included files
   1 while $self->_end_include;
 
-  if ( reftype( $self->_get('main_source') ) eq '' ) {
+  my $ms = $self->_get('main_source');
+  if ( defined( $ms ) && ( reftype( $ms ) // '' ) eq '' ) {
 
     # the main source was passed as a scalar, so we opened its
     # filehandle
@@ -480,20 +490,24 @@ sub READLINE {
 
         # the path is relative; it is relative to the directory of the
         # including file
-        $path = file( file( $self->_get('source') )->parent, $path )->resolve;
+        $path = file( file( $self->_get('source') )->parent, $path );
       }
-      CORE::open my $newifh, '<', "$path"
-        or croak "Cannot open include file '$path': $!";
-      my $suffix = substr( $line, $+[0] );    # text beyond the regex match
-      push @{ $self->_get('suffixes') }, $suffix;    # save for later
+      # avoid infinite recursion
+      my $sources = $self->_get('sources');
+      if ( none { $_ eq $path } @{ $sources } ) {
+        if ( CORE::open my $newifh, '<', "$path" ) {
+          my $suffix = substr( $line, $+[0] ); # text beyond the regex match
+          push @{ $self->_get('suffixes') }, $suffix; # save for later
 
-      push @{ $self->_get('ifhs') },    $self->_get('ifh');     # save for later
-      push @{ $self->_get('sources') }, $self->_get('source');  # save for later
+          push @{ $self->_get('ifhs') }, $self->_get('ifh'); # save for later
+          push @{ $sources }, $self->_get('source'); # save for later
 
-      $self->_set( ifh => $newifh )    # current source is included file
-        ->_set( source => $path );     # current source
-      $line = substr( $line, 0, $-[0] )    # text before the regex match
-        . $self->READLINE;    # append first line from included file
+          $self->_set( ifh => $newifh ) # current source is included file
+            ->_set( source => $path );  # current source
+          $line = substr( $line, 0, $-[0] ) # text before the regex match
+            . $self->READLINE;  # append first line from included file
+        }                       # otherwise we leave the original text
+      }
     }
     return $line;
   }
@@ -548,7 +562,8 @@ sub open {
     or reftype($regex) ne 'REGEXP';
   croak "Transform, if set, must be a code reference"
     if $coderef and reftype($coderef) ne 'CODE';
-  $self->_set( source => $source )->_set( main_source => $source )
+  $self->_set( source => file($source)->absolute )
+    ->_set( main_source => $source )
     ->_set( include => $regex )->_set( transform => $coderef );
   return $self;
 }
@@ -609,7 +624,7 @@ into the C<$buffer> at offset C<$offset>, similar to the
 L<CORE::read|perlfunc/read> function.  Returns the number of
 characters read, or 0 when there are no more characters.
 
-If L</set_read_by_lines> is active, then the reading stops after the
+If L</set_read_by_line> is active, then the reading stops after the
 first encountered input record separator (L<$E<sol>|perlvar/"$/">),
 even if the requested number of characters has not been reached yet.
 
@@ -852,7 +867,7 @@ position, or the very end of the stream.
 
 =item
 
-L</tell> cannot be used on an B<IO::ReadHandle::Include>.
+L<tell> cannot be used on an B<IO::ReadHandle::Include>.
 
 =back
 
